/*
 * Example for iqRF click
 *
 * Date         Mar 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration AVR :
 *  MCU           :        ATMEGA32
 *  Dev. Board    :        EasyAVR v7
 *  SW            :        mikroC PRO for AVR v6.1.1
 *  NOTE          : - This file contains code for both transmitter and receiver.
 *                     It demonstrates sending and receiving of RF packet using
 *                     UART with 9600 baud rate after given triggering point
 *                     measured with Thermo K sensor
 *                  - Connect LCD and switch SW3.1 into right position
 *
 *   TX Setting   : - Place iqRF click board on mikroBUS slot 1
 *                  - Place Thermo K click board on mikroBUS slot 2
 *                  - If temperature from sensor reaches its triggering point,
 *                     send RF packet to Rx by writing data to UART
 *
 *   RX Setting   : - Place iqRF click board on mikroBUS slot 1
 *                  - Observe alert messages on display
 *
 ******************************************************************************/

#define __RX__                          // Choose between __TX__ and __RX__

#include <stdbool.h>

bool low_point_reached = false;
bool high_point_reached = false;

sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;

#ifdef __TX__

#include "MCP9600.h"

// Choose triggering temperature points
#define LOW_TEMP_POINT 20
#define HIGH_TEMP_POINT 30

char _status;
char txt[10];
char lab1[10];
char old_txt[10];
char tmp_data[10];
char Temp_Data_Ready;

/*******************************************************************************
* Function Write_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function writes Data to MCP9600
* Input: Register address, data
* Output: Nothing
*******************************************************************************/
char Write_MCP9600_Data(char RegAddr, char _data)
{
  tmp_data[0] = RegAddr;                                                        // Register address
  tmp_data[1] = _data;

  TWI_Start();
  TWI_Write( MCP9600_I2C_ADDR );
  TWI_Write( tmp_data[ 0 ] );
  TWI_Write( tmp_data[ 1 ] );
  TWI_Stop();
}

/*******************************************************************************
* Function Read_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Data( char RegAddr )
{
  tmp_data[0] = RegAddr;                                                        // Register address

  TWI_Start();
  TWI_Write( MCP9600_I2C_ADDR );
  TWI_Write( tmp_data[ 0 ] );
  TWI_Stop();
  Delay_us( 50 );
  TWI_Start();
  TWI_Write( MCP9600_I2C_ADDR | 1 );
  tmp_data[ 0 ] = TWI_Read( 0 );
  TWI_Stop();

  return tmp_data[0];                                                           // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_MCP9600_Status
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Status()
{
  char DataSum;

  tmp_data[0] = MCP9600_Status;                                                 // Register address

  TWI_Start();
  TWI_Write( MCP9600_I2C_ADDR );
  TWI_Write( tmp_data[ 0 ] );
  TWI_Stop();
  Delay_us( 50 );
  TWI_Start();
  TWI_Write( MCP9600_I2C_ADDR | 1 );
  tmp_data[ 0 ] = TWI_Read( 0 );
  TWI_Stop();
  DataSum = tmp_data[0];

  return DataSum;                                                               // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_Temperature()
* ------------------------------------------------------------------------------
* Overview: Function gets Thermocouple Temperature from MCP9600
* Input: Nothing
* Output: Thermocouple Temperature
*******************************************************************************/
float Read_Temperature()
{
  float Temperature;

  tmp_data[0] = MCP9600_TH;                                                     // Thermocouple Temperature register

  TWI_Start();
  TWI_Write( MCP9600_I2C_ADDR );
  TWI_Write( tmp_data[ 0 ] );
  TWI_Stop();
  Delay_us( 50 );
  TWI_Start();
  TWI_Write( MCP9600_I2C_ADDR | 1 );
  tmp_data[ 0 ] = TWI_Read( 1 );
  tmp_data[ 1 ] = TWI_Read( 0 );
  TWI_Stop();

  if((tmp_data[0] & 0x80) == 0x80){                                             // TA < 0 C
    tmp_data[0] = tmp_data[0] & 0x7F;                                           // Clear SIGN
    Temperature = 1024 - (tmp_data[0]*16 + tmp_data[1] / 16);
  }
  else                                                                          // TA > 0 C
    Temperature = (tmp_data[0] * 16 + (float)tmp_data[1] / 16);                 // Temperature = Ambient Temperature (C)

  return Temperature;                                                           // Return Thermocouple temperature data
}
#endif // __TX__

//******************************************************************************
// Main function
//******************************************************************************

void main()
{
  char res;
  float temp;

  Lcd_Init();                        // Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Out( 1, 4, "iqRF click" );

  #ifdef __TX__
  // Initialize the I2C1 module with 100Kbps speed on PORTB
  TWI_Init( 100000 );
  Delay_ms( 200 );
  #endif
  UART1_Init( 9600 );                                                           // Initialize UART
  Delay_ms(100);

  #ifdef __RX__
////////////////////////////////////////////////////////////////////////////////
  UCSRB |= ( 1 << RXCIE );
  SREG  |= ( 1 << SREG_I );
  #endif
////////////////////////////////////////////////////////////////////////////////

  while(1)
  {
    #ifdef __TX__
    ///////////////////////////////////////////////////////////////////////////
    _status = &Read_MCP9600_Status;
    Temp_Data_Ready = (_status & 0x40);

    temp =  Read_Temperature();

    if (Temp_Data_Ready)
    {                                                                           // Check is Temperature Data ready
      sprintf(txt, "%2.1f  C", temp);                                           // Format Temperature Data and store it to txt
      res = strcmp(txt,old_txt);                                                // Compare old_txt and txt

      if(res != 0)                                                              // If old_txt and txt are not equal
      {
        Lcd_Out(2, 6, "      ");
        Lcd_Out(2, 6, txt);                                                     // Write temperature value on display
      }
    }

    if (temp <= LOW_TEMP_POINT)                                                 // If low temperature point has been reached
    {
     if (!low_point_reached)
     {
      UART1_Write_Text("L");                                                    // Send byte via UART
     }
     low_point_reached = true;

    }
    else if (temp >= HIGH_TEMP_POINT)                                           // If high temperature point has been reached
    {
     if(!high_point_reached)
     {
      UART1_Write_Text("H");                                                    // Send byte via UART
     }
     high_point_reached = true;

    }
    else
    {
     low_point_reached = false;
     high_point_reached = false;
    }
    Delay_ms(500);
  #endif
  //////////////////////////////////////////////////////////////////////////////
  #ifdef __RX__

    // Writing warnings on display
    if(high_point_reached)
    {
     high_point_reached = false;
     Lcd_Out(2, 3, "HIGH REACHED");
     Delay_ms(2000);
     Lcd_Out(2, 3, "             ");
    }
    if(low_point_reached)
    {
     low_point_reached = false;
     Lcd_Out(2, 3, "LOW REACHED");
     Delay_ms(2000);
     Lcd_Out(2, 3, "             ");
    }
  #endif
  }
}

#ifdef __RX__
////////////////////////////////////////////////////////////////////////////////
/*******************************************************************************
* Function uart_rx_ISR()
* ------------------------------------------------------------------------------
* Overview: UART Interrupt InService Routine for receiving UART data from IQRF MCU
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void uart_rx_ISR() iv IVT_ADDR_USART__RXC ics ICS_AUTO
{
    char tmp = UART_Read();

        if( tmp == 'L')
        {
         low_point_reached = true;
         high_point_reached = false;
        }
        else if( tmp == 'H')
        {
         high_point_reached = true;
         low_point_reached = false;
        }
}
#endif
//////////////////////////////////////////////////////////////////END OF FILE///